#include <STC15F2K60S2.H>//STC15系列单片机驱动文件
#include "./Timer.h"//硬件定时器库
#include "./SMG.h"//数码管库
#include "./DS18B20.h"//DS18B20库
#include "./ONEWIRE.h"//软件单总线库
#include "./Sys.h"//系统库
#include "./config.h"//全局配置库
#include "intrins.h"//KEIL内部库
#include "string.h"//字符串库
#include "math.h"//数学库

//采集温度需要较快响应,存放于IDATA区域中可以更快操作
DS18B20_Temp_TypeDef idata ds18b20_temp;//定义句柄

//DS18B20变量初始化函数
void DS18B20_Tag_Init()
{
	ds18b20_temp.Temp_H=0x00;//温度值高8位
	ds18b20_temp.Temp_L=0x00;//温度值低8位
	ds18b20_temp.Temp=0x0000;  //温度值16位	
}

//温度采集函数
/***顺序必须遵循:初始化->ROM指令->函数指令***/
/***上电默认85°C,需要采取软件延时跳过***/
float Tem_Capture()
{
	float Temp_Val;//浮点型数据,存储温度值(带小数)
	long temp;//温度数据中转变量
	
	EA=0;//关闭总中断,以防中断扰乱AD转换,温度值出错
	/*****1*****/
	init_ds18b20();//初始化DS18B20
	//忽略ROM指令,对总线上所有的器件进行操作
	Write_DS18B20(SKIP_ROM);
	//启动温度转换指令,DS18B20启动转换温度动作,并将热数据
	//存放于剪切板中的第0和第1字节
	Write_DS18B20(CONVERT_T);
	
	/*****2*****/
	init_ds18b20();//初始化DS18B20
	//忽略ROM指令,对总线上所有的器件进行操作
	Write_DS18B20(SKIP_ROM);
	//读取剪切板指令,温度数据16位在第0和第1字节中
	Write_DS18B20(READ_SCRATCHPAD);
	ds18b20_temp.Temp_L=Read_DS18B20();//读取温度值低8位
	ds18b20_temp.Temp_H=Read_DS18B20();//读取温度值高8位
	
	/*****3*****/
	//计算出16位热数据,计算出温度值
	Temp_Val=ds18b20_temp.Temp_H*256+ds18b20_temp.Temp_L;
	temp=Temp_Val;
	if(temp&0Xf800)//负数
	{
		// 负温度处理
		temp = ~temp + 1; // 取反加1，得到绝对值
		Temp_Val = (float)temp * 0.0625;//温度最小位是2^-4,要乘以2^-4来处理数据
		Temp_Val = -Temp_Val; // 取负号
	}
	
	else
	{
		Temp_Val=Temp_Val*0.0625;//温度最小位是2^-4,要乘以2^-4来处理数据
		Temp_Val=Temp_Val*10000;//保留4位小数,所以这么处理
	}
	
	EA=1;//重新打开总中断,准备下次转换
	
	return (float)Temp_Val;//返回浮点型温度数据
}